#!/usr/bin/env python
import logging
from os.path import abspath, dirname, join

from edc_constants.constants import DM, HIV, HTN
from edc_test_utils import DefaultTestSettings, func_main

base_dir = dirname(abspath(__file__))
app_name = "edc_dx"

project_settings = DefaultTestSettings(
    calling_file=__file__,
    BASE_DIR=base_dir,
    APP_NAME=app_name,
    ETC_DIR=join(base_dir, app_name, "tests", "etc"),
    SUBJECT_SCREENING_MODEL="edc_metadata.subjectscreening",
    SUBJECT_CONSENT_MODEL="edc_metadata.subjectconsent",
    SUBJECT_VISIT_MODEL="edc_visit_tracking.subjectvisit",
    SUBJECT_VISIT_MISSED_MODEL="edc_metadata.subjectvisitmissed",
    SUBJECT_REQUISITION_MODEL="edc_metadata.subjectrequisition",
    LIST_MODEL_APP_LABEL="edc_dx",
    REPORT_DATETIME_FIELD_NAME="report_datetime",
    EDC_DX_REVIEW_LIST_MODEL_APP_LABEL="dx_app",
    EDC_DX_REVIEW_APP_LABEL="dx_app",
    EDC_DX_LABELS={
        HIV: HIV,
        DM: "Diabetes",
        HTN: "Hypertension",
    },
    INSTALLED_APPS=[
        "django.contrib.admin",
        "django.contrib.auth",
        "django.contrib.contenttypes",
        "django.contrib.sessions",
        "django.contrib.messages",
        "django.contrib.staticfiles",
        "django.contrib.sites",
        "django_crypto_fields.apps.AppConfig",
        "django_revision.apps.AppConfig",
        "multisite",
        "edc_appointment.apps.AppConfig",
        "edc_action_item.apps.AppConfig",
        "edc_crf.apps.AppConfig",
        "edc_device.apps.AppConfig",
        "edc_facility.apps.AppConfig",
        "edc_lab.apps.AppConfig",
        "edc_list_data.apps.AppConfig",
        "edc_metadata.apps.AppConfig",
        "edc_offstudy.apps.AppConfig",
        "edc_registration.apps.AppConfig",
        "edc_identifier.apps.AppConfig",
        "edc_notification.apps.AppConfig",
        "edc_sites.apps.AppConfig",
        "edc_timepoint.apps.AppConfig",
        "edc_visit_schedule.apps.AppConfig",
        "edc_visit_tracking.apps.AppConfig",
        "edc_dx_review.apps.AppConfig",
        "edc_dx.apps.AppConfig",
        "dx_app.apps.AppConfig",
    ],
    RANDOMIZATION_LIST_PATH=join(base_dir, app_name, "tests", "test_randomization_list.csv"),
    add_dashboard_middleware=True,
    use_test_urls=True,
).settings


def main():
    func_main(project_settings, f"{app_name}.tests")


if __name__ == "__main__":
    logging.basicConfig()
    main()



from django.contrib.sites.models import Site
from edc_constants.constants import NO
from model_bakery.recipe import Recipe

from dx_app.models import (
    ClinicalReviewBaseline,
    DmInitialReview,
    HivInitialReview,
    HtnInitialReview,
)

clinicalreviewbaseline = Recipe(
    ClinicalReviewBaseline,
    site=Site.objects.get_current(),
    hiv_dx=NO,
    htn_dx=NO,
    dm_dx=NO,
)


hivinitialreview = Recipe(HivInitialReview, dx_ago=None, dx_date=None)
htninitialreview = Recipe(HtnInitialReview, dx_ago=None, dx_date=None)
dminitialreview = Recipe(DmInitialReview, dx_ago=None, dx_date=None)



from importlib.metadata import version

__version__ = version("edc_dx")

from .diagnoses import Diagnoses
from .utils import (
    get_diagnosis_labels,
    get_diagnosis_labels_prefixes,
    raise_on_unknown_diagnosis_labels,
)



from django.contrib import admin
from django.urls import path
from django.views.generic import RedirectView

app_name = "edc_dx"

urlpatterns = [
    path("admin/", admin.site.urls),
    path("", RedirectView.as_view(url="admin/"), name="home_url"),
]



from django.test import TestCase, override_settings
from edc_appointment.constants import INCOMPLETE_APPT
from edc_constants.constants import DM, HIV, HTN, NO, YES
from model_bakery import baker

from edc_dx.diagnoses import (
    ClinicalReviewBaselineRequired,
    Diagnoses,
    InitialReviewRequired,
    MultipleInitialReviewsExist,
)

from ..test_case_mixin import TestCaseMixin


class TestDiagnoses(TestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_identifier = self.enroll()
        self.create_visits(self.subject_identifier)

    def test_diagnoses_no_baseline_review_raises(self):
        self.assertRaises(
            ClinicalReviewBaselineRequired,
            Diagnoses,
            subject_identifier=self.subject_visit_baseline.subject_identifier,
        )

        clinical_review_baseline = baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            dm_dx=NO,
            htn_dx=NO,
        )
        try:
            diagnoses = Diagnoses(
                subject_identifier=self.subject_visit_baseline.subject_identifier,
            )
        except ClinicalReviewBaselineRequired:
            self.fail("DiagnosesError unexpectedly raised")

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNone(diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(DM))

        clinical_review_baseline.htn_dx = YES
        clinical_review_baseline.save()

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
        )
        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertEqual(YES, diagnoses.get_dx(HTN))
        self.assertIsNone(diagnoses.get_dx(DM))

        clinical_review_baseline.dm_dx = YES
        clinical_review_baseline.save()

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
        )
        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertEqual(YES, diagnoses.get_dx(HTN))
        self.assertEqual(YES, diagnoses.get_dx(DM))

    def test_diagnoses_dates_baseline_raises(self):
        """Assert expects the initial review model instance before
        returning a dx.
        """

        for prefix in [HIV, DM, HTN]:
            opts = {
                "subject_visit": self.subject_visit_baseline,
                "hiv_dx": NO,
                "dm_dx": NO,
                "htn_dx": NO,
            }
            prefix = prefix.lower()
            opts.update({f"{prefix}_dx": YES})
            obj = baker.make("dx_app.clinicalreviewbaseline", **opts)
            diagnoses = Diagnoses(
                subject_identifier=self.subject_visit_baseline.subject_identifier,
            )
            self.assertRaises(InitialReviewRequired, diagnoses.get_dx_date, HIV)
            obj.delete()

    def test_diagnoses_dates_baseline(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            dm_dx=NO,
            htn_dx=NO,
        )
        baker.make(
            "dx_app.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )
        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNotNone(diagnoses.get_dx_date(HIV))
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
            report_datetime=self.subject_visit_baseline.report_datetime,
            lte=True,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNotNone(diagnoses.get_dx_date(HIV))
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
            report_datetime=self.subject_visit_baseline.report_datetime,
            lte=False,
        )

        self.assertEqual(YES, diagnoses.get_dx(HIV))
        self.assertIsNotNone(diagnoses.get_dx_date(HIV))
        self.assertIsNone(diagnoses.get_dx_date(DM))
        self.assertIsNone(diagnoses.get_dx_date(HTN))

    def test_diagnoses_dates(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            dm_dx=NO,
            htn_dx=NO,
        )

        hiv_initial_review = baker.make(
            "dx_app.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )

        self.subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        self.subject_visit_baseline.appointment.save()
        self.subject_visit_baseline.appointment.refresh_from_db()
        self.subject_visit_baseline.refresh_from_db()

        baker.make(
            "dx_app.clinicalreview",
            subject_visit=self.subject_visit_followup,
            hiv_dx=NO,
            htn_dx=YES,
            dm_dx=NO,
        )

        htn_initial_review = baker.make(
            "dx_app.htninitialreview",
            subject_visit=self.subject_visit_followup,
            dx_ago=None,
            dx_date=self.subject_visit_followup.report_datetime,
        )

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_followup.subject_identifier,
            report_datetime=self.subject_visit_followup.report_datetime,
            lte=True,
        )
        self.assertIsNotNone(diagnoses.get_dx_date(HIV))
        self.assertEqual(
            diagnoses.get_dx_date(HIV),
            hiv_initial_review.get_best_dx_date().date(),
        )

        self.assertEqual(
            diagnoses.get_dx_date(HTN),
            htn_initial_review.get_best_dx_date().date(),
        )
        self.assertIsNotNone(diagnoses.get_dx_date(HTN))

    def test_diagnoses_dates_baseline2(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            htn_dx=NO,
            dm_dx=NO,
        )
        baker.make(
            "dx_app.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )
        self.subject_visit_baseline.appointment.appt_status = INCOMPLETE_APPT
        self.subject_visit_baseline.appointment.save()
        self.subject_visit_baseline.appointment.refresh_from_db()
        self.subject_visit_baseline.refresh_from_db()

    @override_settings(
        EDC_DX_REVIEW_EXTRA_ATTRS={"initialreview": "initialreviewmissingsingleton"}
    )
    def test_diagnoses_dates_baseline3(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            htn_dx=NO,
            dm_dx=NO,
        )

        baker.make(
            "dx_app.hivinitialreviewmissingsingleton",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )

        baker.make(
            "dx_app.hivinitialreviewmissingsingleton",
            subject_visit=self.subject_visit_followup,
            dx_ago="5y",
            rx_init_ago="4y",
        )

        diagnoses = Diagnoses(
            subject_identifier=self.subject_visit_baseline.subject_identifier,
        )
        self.assertRaises(MultipleInitialReviewsExist, getattr, diagnoses, "initial_reviews")



from django import forms
from edc_crf.modelform_mixins import CrfModelFormMixin
from edc_dx_review.forms.clinical_review_baseline_form import (
    ClinicalReviewBaselineFormValidator,
)
from edc_dx_review.models import ClinicalReviewBaseline


class ClinicalReviewBaselineForm(CrfModelFormMixin, forms.ModelForm):
    form_validator_cls = ClinicalReviewBaselineFormValidator

    def validate_against_consent(self):
        """Skip for tests"""
        pass

    class Meta:
        model = ClinicalReviewBaseline
        fields = "__all__"



from django.test import TestCase, override_settings
from edc_constants.constants import DM, HIV

from edc_dx import get_diagnosis_labels_prefixes


class TestLabels(TestCase):
    @override_settings(EDC_DX_LABELS={HIV: "HIV"})
    def test_diagnosis_labels_hiv_only(self):
        self.assertEqual([x.lower() for x in [HIV]], get_diagnosis_labels_prefixes())

    @override_settings(EDC_DX_LABELS={HIV: "HIV", DM: "Diabetes"})
    def test_diagnosis_labels_hiv_dm_only(self):
        self.assertEqual([x.lower() for x in [HIV, DM]], get_diagnosis_labels_prefixes())



from django import forms
from django.test import TestCase
from edc_constants.constants import DM, HIV, HTN, NO, NOT_APPLICABLE, YES
from edc_crf.crf_form_validator_mixins import CrfFormValidatorMixin
from edc_form_validators import FormValidator
from model_bakery import baker

from dx_app.models import ClinicalReviewBaseline
from edc_dx import get_diagnosis_labels
from edc_dx.form_validators import DiagnosisFormValidatorMixin

from ..test_case_mixin import TestCaseMixin


class DiagnosisFormValidator(
    CrfFormValidatorMixin, DiagnosisFormValidatorMixin, FormValidator
):
    def clean(self):
        self.get_diagnoses()
        for prefix, label in get_diagnosis_labels():
            self.applicable_if_not_diagnosed(
                prefix=prefix,
                field_applicable=f"{prefix}_test",
                label=label,
            )

    def get_consent_for_period_or_raise(self):
        pass


class MyModel:
    @classmethod
    def related_visit_model_attr(cls):
        return "subject_visit"


class TestDiagnosisFormValidator(TestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_identifier = self.enroll()
        self.create_visits(self.subject_identifier)

    def test_ok(self):
        data = dict(subject_visit=self.subject_visit_followup)
        form_validator = DiagnosisFormValidator(cleaned_data=data, model=MyModel)
        self.assertRaises(forms.ValidationError, form_validator.validate)
        self.assertIn(
            f"Please complete {ClinicalReviewBaseline._meta.verbose_name}",
            str(form_validator._errors.get("__all__")),
        )

    def test_ok2(self):
        data = dict(subject_visit=self.subject_visit_followup)
        form_validator = DiagnosisFormValidator(cleaned_data=data, model=MyModel)
        self.assertRaises(forms.ValidationError, form_validator.validate)
        self.assertIn(
            f"Please complete {ClinicalReviewBaseline._meta.verbose_name}",
            str(form_validator._errors.get("__all__")),
        )


class ApplicableIfDiagnosedFormValidator(
    DiagnosisFormValidatorMixin, CrfFormValidatorMixin, FormValidator
):
    def clean(self):
        for prefix, label in get_diagnosis_labels().items():
            self.applicable_if_diagnosed(
                diagnoses=self.get_diagnoses(),
                prefix=prefix,
                field_applicable=f"{prefix}_test",
                label=label,
            )


class TestApplicableIfDiagnosedFormValidation(TestCaseMixin, TestCase):
    def setUp(self):
        super().setUp()
        self.subject_identifier = self.enroll()
        self.create_visits(self.subject_identifier)

    def test_no_dx_ok(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=NO,
            dm_dx=NO,
            htn_dx=NO,
        )
        cleaned_data = {
            "subject_visit": self.subject_visit_baseline,
            "hiv_test": NOT_APPLICABLE,
            "dm_test": NOT_APPLICABLE,
            "htn_test": NOT_APPLICABLE,
        }
        form_validator = ApplicableIfDiagnosedFormValidator(
            cleaned_data=cleaned_data, model=MyModel
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_no_dx_with_answer_raises(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=NO,
            dm_dx=NO,
            htn_dx=NO,
        )

        for cond in [HIV.lower(), DM, HTN]:
            for answer in [YES, NO]:
                with self.subTest(cond=cond, answer=answer):
                    cleaned_data = {
                        "subject_visit": self.subject_visit_baseline,
                        "hiv_test": NOT_APPLICABLE,
                        "dm_test": NOT_APPLICABLE,
                        "htn_test": NOT_APPLICABLE,
                    }

                    cleaned_data.update({f"{cond}_test": answer})

                    form_validator = ApplicableIfDiagnosedFormValidator(
                        cleaned_data=cleaned_data, model=MyModel
                    )
                    with self.assertRaises(forms.ValidationError) as cm:
                        form_validator.validate()
                    self.assertIn(f"{cond}_test", cm.exception.error_dict)
                    self.assertIn(
                        "Patient was not previously diagnosed with "
                        f"{get_diagnosis_labels().get(cond)}.",
                        str(cm.exception.error_dict.get(f"{cond}_test")),
                    )

    def test_hiv_dx_yes_requires_answer(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            dm_dx=NO,
            htn_dx=NO,
        )
        baker.make(
            "dx_app.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )
        cleaned_data = {
            "subject_visit": self.subject_visit_baseline,
            "hiv_test": NOT_APPLICABLE,
            "dm_test": NOT_APPLICABLE,
            "htn_test": NOT_APPLICABLE,
        }
        form_validator = ApplicableIfDiagnosedFormValidator(
            cleaned_data=cleaned_data, model=MyModel
        )
        with self.assertRaises(forms.ValidationError) as cm:
            form_validator.validate()
        self.assertIn("hiv_test", cm.exception.error_dict)
        self.assertIn(
            "Patient was previously diagnosed with HIV. Expected YES or NO.",
            str(cm.exception.error_dict.get("hiv_test")),
        )

        for answer in [YES, NO]:
            with self.subTest(answer=answer):
                cleaned_data.update({"hiv_test": answer})
                form_validator = ApplicableIfDiagnosedFormValidator(
                    cleaned_data=cleaned_data, model=MyModel
                )
                try:
                    form_validator.validate()
                except forms.ValidationError as e:
                    self.fail(f"ValidationError unexpectedly raised. Got {e}")

    def test_all_dx_yes_requires_answer(self):
        baker.make(
            "dx_app.clinicalreviewbaseline",
            subject_visit=self.subject_visit_baseline,
            hiv_dx=YES,
            dm_dx=YES,
            htn_dx=YES,
        )
        baker.make(
            "dx_app.hivinitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="5y",
            rx_init_ago="4y",
        )
        baker.make(
            "dx_app.dminitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="2y",
            rx_init_ago="1y",
        )
        baker.make(
            "dx_app.htninitialreview",
            subject_visit=self.subject_visit_baseline,
            dx_ago="1y",
            rx_init_ago="1y",
        )

        cleaned_data = {
            "subject_visit": self.subject_visit_baseline,
            "hiv_test": YES,
            "dm_test": YES,
            "htn_test": YES,
        }
        form_validator = ApplicableIfDiagnosedFormValidator(
            cleaned_data=cleaned_data, model=MyModel
        )
        try:
            form_validator.validate()
        except forms.ValidationError as e:
            self.fail(f"ValidationError unexpectedly raised. Got {e}")

        for cond in [HIV.lower(), DM, HTN]:
            with self.subTest(cond=cond):
                cleaned_data = {
                    "subject_visit": self.subject_visit_baseline,
                    "hiv_test": YES,
                    "dm_test": YES,
                    "htn_test": YES,
                }
                cleaned_data.update({f"{cond}_test": NOT_APPLICABLE})
                form_validator = ApplicableIfDiagnosedFormValidator(
                    cleaned_data=cleaned_data, model=MyModel
                )
                with self.assertRaises(forms.ValidationError) as cm:
                    form_validator.validate()
                self.assertIn(f"{cond}_test", cm.exception.error_dict)
                self.assertIn(
                    f"Patient was previously diagnosed with "
                    f"{get_diagnosis_labels().get(cond)}. Expected YES or NO.",
                    str(cm.exception.error_dict.get(f"{cond}_test")),
                )



from django.db import models
from django.db.models import PROTECT
from edc_crf.model_mixins import SingletonCrfModelMixin
from edc_dx_review.model_mixins import (
    ClinicalReviewBaselineModelMixin,
    ClinicalReviewModelMixin,
    dx_initial_review_model_mixin_factory,
    rx_initial_review_model_mixin_factory,
)
from edc_dx_review.model_mixins.factory import (
    baseline_review_model_mixin_factory,
    followup_review_model_mixin_factory,
)
from edc_model.models import BaseUuidModel
from edc_utils import get_utcnow
from edc_visit_tracking.models import SubjectVisit


class ClinicalReviewBaseline(
    baseline_review_model_mixin_factory(),
    ClinicalReviewBaselineModelMixin,
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @classmethod
    def related_visit_model_attr(cls):
        return "subject_visit"

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class ClinicalReview(
    followup_review_model_mixin_factory(),
    ClinicalReviewModelMixin,
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class DmInitialReview(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="medicines for diabetes"
    ),
    SingletonCrfModelMixin,
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class HtnInitialReview(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="medicines for hypertension"
    ),
    SingletonCrfModelMixin,
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class HivInitialReview(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="antiretroviral therapy (ART)"
    ),
    SingletonCrfModelMixin,
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class DmInitialReviewMissingSingleton(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="medicines for diabetes"
    ),
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class HtnInitialReviewMissingSingleton(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="medicines for hypertension"
    ),
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier


class HivInitialReviewMissingSingleton(
    dx_initial_review_model_mixin_factory(),
    rx_initial_review_model_mixin_factory(
        "rx_init", verbose_name_label="antiretroviral therapy (ART)"
    ),
    BaseUuidModel,
):
    subject_visit = models.ForeignKey(SubjectVisit, on_delete=PROTECT)

    report_datetime = models.DateTimeField(default=get_utcnow)

    @property
    def related_visit(self):
        return self.subject_visit

    @property
    def subject_identifier(self):
        return self.subject_visit.subject_identifier
